//iPod RSVP
//Adam Ferguson

#include <stdio.h>                                                         //Needed for basic I/O
#include <string.h>                                                        //Needed for various string operations
#include <errno.h>                                                         //Needed for error reporting
#include <dirent.h>                                                        //Needed for listing files in /mnt/Notes directory
#include "pz.h"                                                            //Needed for Podzilla operations

static PzModule *module;                                                   //The module itself           
static PzConfig *ipodRSVP_config = NULL;                                   //The configuration for the module
static PzWidget *slide;                                                    //The 'display speed' selection slider
static PzWidget *text_area;                                                //The text area that holds the current word
static PzWindow *displaySpeedWindow;                                       //The 'display speed' selection window
static ttk_timer MainTimer;                                                //The timer for changing words

//Global Settings & State Variables
#define CURRENT_FILE 1                                                     //These are for the podzilla
#define BOOKMARK 2                                                         //settings sids
#define DISPLAY_SPEED 3
#define PAUSE_COMMA 4
#define PAUSE_MISC_PUNCT 5
#define PAUSE_SENTENCE 6
#define PAUSE_LENGTH 7

static char firstEntry[255] = "/Extras/Utilities/iPod RSVP/No Books Found";//The first menu item
static char currentWord[255];                                              //The word currently being displayed
static const char* pause_length[] = { "Short", "Medium", "Long", NULL };   //The length of the pauses
static const char *SelectedBook;                                           //The currently selected book's filename
static char *text;                                                         //The full text of the current book
static char *position;                                                     //The position in memory of the start of the current word
static int unix_or_windows = 0;                                            //Is the current file a windows or *nix text file? 0 == *nix
static int *value;                                                         //Current display speed (roughly corresponds to WPM)
static int progress = 0;                                                   //The byte offset (from the beginning) of the current file
static short int longer_delay = 0;                                         //Are we in a special case that warrants a longer-than-usual delay? 0 == no
static short int long_word = 0;                                            //Is the current word long? 0 == no
static short int beginning_of_long_word = 0;                               //Keeps track of whether or not this is the first sighting of a long word
static short int state;                                                    //Is the program reading, or is it paused? 0 == paused
static short int begining_end;                                             //Are we at the beginning of the file, or the very end? -1 == beginning, 1 == end
static short int last_direction = 1;                                       //Was the last move in the file forward or backwards? 0 == backwards
static short int delay = 500;                                              //Number of milliseconds between words (affected by *value)
static short int files_exist = 0;                                          //Are there even any text files in the Notes directory? 0 = no, 1 = yes
/////////////////////////

//Function Prototypes
static void move_word (int direction);                                     //Move the position pointer one word forward or backwards
static void clip_word(int difference);                                     //On some Ipods, it is necessary to shorten the words
static void Update_First_Menu();                                           //Change text of first menu item (start/resume, filename, etc)
static void draw_text();                                                   //Timer function that cycles through the words in the current file
static int ipodRSVP_saveSettings( TWidget *this, int key, int time );      //Save program settings (progress && value)
static int ipodRSVP_saveBook(TWidget *this, int key, int time);            //Save selected book
static PzWindow *new_displaySpeedWindow();                                 //Initialize 'display speed' window
static PzWindow *new_selectBook();                                         //Initialize 'select book' window  
static PzWindow *new_ipodRSVP_window ();                                   //Initialize main text window
static void draw_ipodRSVP (PzWidget * wid, ttk_surface srf);               //Draw the window that holds the current word
static void draw_displaySpeedWindow (PzWidget *wid, ttk_surface srf);      //Draw the window that sets the display speed
static int event_displaySpeedWindow (PzEvent *ev);                         //Handle events from the 'display speed' window
static int event_ipodRSVP (PzEvent * ev);                                  //Handle events from the main text window
static void cleanup_ipodRSVP ();                                           //Cleanup for when Podzilla ends
static void init_ipodRSVP ();                                              //Initialize main program settings, load settings, etc.

static void move_word (int direction)
{
  char backupCharacter = 0;                                                //Stores the ' ' character
  char *space_pos = 0;                                                     //Location of the next space
  char *lf_pos = 0;                                                        //Location of the next line-feed character
  char *replace_pos = 0;                                                   //Location of the character to replace
  int difference = 0;                                                      //Length of the current word
  int x = 0;                                                               //Counting variable


  if (direction && begining_end != 1)                                      //If we're moving forward and we're not at the first word
  {
    space_pos = strchr (position, ' ');                                    //Find location of next space
    lf_pos = strchr (position, '\n');                                      //Find location of next line-feed
    (space_pos < lf_pos) ? (replace_pos = space_pos) : (replace_pos = lf_pos); //Store the lesser of those two in replace_pos
    if (space_pos == 0 && lf_pos != 0)                                     //If there are no spaces, but there is a line-feed...
	{
	  replace_pos = lf_pos;                                                //Store the location of the line-feed
	}
    if (lf_pos == 0 && space_pos != 0)                                     //If there are no line-feeds, but there is a space...
	{
	  replace_pos = space_pos;                                             //Store the space
	}
    difference = (replace_pos) - (position);                               //Store the length of the current word
    begining_end = 0;                                                      //We are not at the beginning of text
    if (difference < 0)                                                    //If the word length is less than zero...
	{
	  difference = strlen (position);                                      //The word length is the length of the remaining text (i.e., the last word)
	  state = 0;                                                           //Stop playback
	  begining_end = 1;                                                    //We are at the end of text
	}
	for (x = 0; x < difference; x++)
	{
	  currentWord[x] = position[x];                                        //Copy each character from the current word (in the text stream) into currentWord
	}
    currentWord[difference] = 0;                                           //Set the last character to 0 for the end of the string
    if(ttk_get_podversion() <= 20 ){clip_word(difference);}                //Shorten long words on older iPods
	if((ttk_get_podversion() <= 20) && difference > 12){position += strlen(currentWord);} //If we have a long word on an old ipod, don't skip the next character
	else{position += strlen(currentWord) + 1;}                             //Otherwise, advance to the beginning of the next word
	while(position[0] == '\n'){position+=1;}
	progress = position - text;                                            //Update the progress variable with the current offset
	if(pz_menu_get_option("/Extras/Utilities/iPod RSVP/Pause after.../Commas") && currentWord[difference-1] == ',' ){longer_delay = 1;}  //Pause longer for commas
	if(pz_menu_get_option("/Extras/Utilities/iPod RSVP/Pause after.../Misc. punctuation") && (currentWord[difference-1] == ';' || currentWord[difference-1] == '-' || currentWord[difference-1] == '"')){longer_delay = 1;} //Pause longer for semi-colons and other misc punctuation
	if(pz_menu_get_option("/Extras/Utilities/iPod RSVP/Pause after.../End of sentences"))
	{
		if(currentWord[difference-1] == '.' || currentWord[difference-1] == '!' || currentWord[difference-1] == '?')  //if the last character of the word is a punctuation mark
		{
			if(strcmp(currentWord,"Mr.") != 0 
			   && strcmp(currentWord, "Ms.") != 0                          //These conditions ensure that only 
			   && strcmp(currentWord, "Mrs.") != 0                         //the ends of sentences are paused.
			   && strcmp(currentWord, "Dr.") != 0                          //Titles are ignored.
			   && strcmp(currentWord, "mr.") != 0
			   && strcmp(currentWord, "ms.") != 0
			   && strcmp(currentWord, "mrs.") != 0
			   && strcmp(currentWord, "dr.") != 0) {longer_delay = 1;
			}
		}
	}
  } 
  else                                                                     //If we're moving backwards...
  {
    if (begining_end != -1)                                                //And if we are not on the last word already...
	{
	  backupCharacter = *(position - 1);                                   //Store the last encountered space
	  *(position - 1) = 0;                                                 //Change the most recent space into a zero to temporarily terminate text
	  space_pos = strrchr (text, ' ');                                     //Now, search for the last space 
	  lf_pos = strrchr (text, '\n');                                       //Search for the last line-feed character
	  (space_pos > lf_pos) ? (replace_pos = space_pos) : (replace_pos = lf_pos);  //Save whichever is closest to the end
	  if (space_pos == 0 && lf_pos != 0)                                   //If there are no spaces, but there is a line-feed...
	  {
	    replace_pos = lf_pos;                                              //Store the line-feed
	  }
	  if (lf_pos == 0 && space_pos != 0)                                   //If there are no line-feeds, but there is a space...
	  {
	    replace_pos = space_pos;                                           //Store the space
	  }
	  difference = position - replace_pos;                                 //Calculate word length
	  if (space_pos == 0 && lf_pos == 0)                                   //If there are no spaces OR line-feeds...
	  {
	    replace_pos = 0;                                                   //We're starting at the beginning
	    difference = -1;                                                   //And set the word length to an impossible value to be caught below
	  }
	  begining_end = 0;                                                    //We are not on the last word now
	  if (difference < 0)                                                  //If the word length is less than zero...
	  {
	    for(x = 0; x < strlen(text); x++){currentWord[x] = text[x];}       //Copy all of the characters from the first word into currentWord
		currentWord[strlen(text)+1] = 0;                                   //Set the last character of currentWord as the null terminator
		*(position - 1) = backupCharacter;                                 //Restore text to its original length
	    position = text;                                                   //Put the position at the beginning of the text
		progress = 0;                                                      //Reset the progress
	    state = 0;                                                         //Stop playback
	    begining_end = -1;                                                 //We are at the very first word
		  if(ttk_get_podversion() <= 20){clip_word(strlen(currentWord));}  //Shorten long words on older iPods
	  }
	  else                                                                 //If the word length is a positive value...
	  {
		for (x = 0; x < difference; x++)
		{
		  currentWord[x] = replace_pos[x + 1];                             //For each character in the current word in the text stream, copy it into currentWord
		}
	    *(position - 1) = backupCharacter;                                 //Restore text to its original length
	    position = replace_pos + 1;                                        //Move position to the previous word
		progress = position - text;                                        //Update the progress variable
		currentWord[difference] = 0;
		if(ttk_get_podversion() <= 20){clip_word(difference);}             //Shorten long words on older iPods
	  }
	}
  }
}

static void clip_word(int difference)
{
	char *hyphen_pos = 0;                                                  //Location of a hyphen in the current word
	if (difference > 12)                                                   //If this is running on an older iPod and we have a long word...
	{
	  long_word = (difference/12)+1;
	  if(beginning_of_long_word == 0){beginning_of_long_word = long_word;} //Set the number of iterations we should expect for this word
	  longer_delay = 1;
      hyphen_pos = strchr(currentWord,'-');                                //See if there are any hyphens in the word (best place to split it)
	  if (hyphen_pos)                                                      //If there are any hyphens...
	  {
		*(hyphen_pos + 1) = 0;                                             //End the word after the hyphen
	  }
	  else                                                                 //If there are no hyphens...
	  {
		currentWord[12] = 0;                                               //End the word at the 12 character mark
	  }
	}
	else{if(long_word){long_word-=1;}                                      //If long_word > 0, subtract 1 from it.
	if(!long_word){beginning_of_long_word = 0;}}                           //When long_word gets to zero, clear the beginning_of_long_word
}

static void Update_First_Menu()
{
  char* location;
  
  SelectedBook = pz_get_string_setting(ipodRSVP_config, CURRENT_FILE);     //Load selected file from the settings
  pz_menu_remove (firstEntry);                                             //Remove top menu entry
  location = strrchr(firstEntry,'/');                                      //Find the last '/' character
  *(location + 1) = 0;                                                     //End the string after the last '/'
  if(progress)                                                             //If there has been some progress in the current book...
  {
	  strcat(firstEntry,"Resume ");                                        //Add the word "Resume" to the front (i.e. /menu/menu/Resume )
	  strcat(firstEntry,SelectedBook);                                     //Add the book's filename to the menu (i.e. /menu/menu/Resume filename.txt)
	  pz_menu_add_top (firstEntry, new_ipodRSVP_window);                   //Add this new menu item to the top
  }
  else                                                                     //If the book has not been started...
  {
	  strcat(firstEntry,"Start ");                                         //Add the word "Start" to the front (i.e. /menu/menu/Start )
	  strcat(firstEntry,SelectedBook);                                     //Add the book's filename to the menu (i.e. /menu/menu/Start filename.txt)
	  pz_menu_add_top (firstEntry, new_ipodRSVP_window);                   //Add this new menu item to the top
  } 
}

static void draw_text ()
{
  if (state)                                                               //If reading is not paused
    {
	  move_word (1);                                                       //Advance the position by one word
      text_area->dirty = 1;                                                //Redraw the window containing the text
      if(longer_delay)                                                     //If we are in a special situation that calls for a longer delay...
	  {
		int delay_multiplier = pz_menu_get_option("/Extras/Utilities/iPod RSVP/Pause Length: "); //Get the setting for the pause length
		MainTimer = ttk_create_timer (delay*(delay_multiplier+2), draw_text);//Set a timer for moving to the next word with a delay that is based on the normal length and the multiplier
		longer_delay = 0;                                                  //Clear the value in longer_delay
	  }
	  else                                                                 //Otherwise...
	  {
		MainTimer = ttk_create_timer (delay, draw_text);                   //Set a timer for moving to the next word with normal delay
	  }
    }
}


static int ipodRSVP_saveSettings( TWidget *this, int key, int time )
{
	if( key == TTK_BUTTON_MENU ) {
		pz_set_int_setting (ipodRSVP_config, DISPLAY_SPEED, *value);       //Set the saved value (roughly equivalent to WPM)
		if(progress < 0){progress = progress * -1;}                        //If for some reason the progress is negative (this shouldn't happen)
		pz_set_int_setting (ipodRSVP_config, BOOKMARK, progress);          //Set the saved value for the progress
		pz_set_int_setting (ipodRSVP_config, PAUSE_COMMA, pz_menu_get_option("/Extras/Utilities/iPod RSVP/Pause after.../Commas"));
		pz_set_int_setting (ipodRSVP_config, PAUSE_MISC_PUNCT, pz_menu_get_option("/Extras/Utilities/iPod RSVP/Pause after.../Misc. punctuation"));
		pz_set_int_setting (ipodRSVP_config, PAUSE_SENTENCE, pz_menu_get_option("/Extras/Utilities/iPod RSVP/Pause after.../End of sentences"));
		pz_set_int_setting (ipodRSVP_config, PAUSE_LENGTH, pz_menu_get_option("/Extras/Utilities/iPod RSVP/Pause Length: "));
		pz_save_config( ipodRSVP_config );                                 //Save the configuration
		Update_First_Menu();                                               //Update the menu to reflect the progress
	} 
	return ttk_menu_button( this, key, time );
}

static int ipodRSVP_saveBook(TWidget *this, int key, int time)
{
	if( key == TTK_BUTTON_MENU ) {                                        
		pz_set_string_setting (ipodRSVP_config, CURRENT_FILE, ttk_menu_get_selected_item (this)->name); //Save the current book's filename
		pz_set_int_setting (ipodRSVP_config, BOOKMARK, 0);                 //Reset progress to 0 (new book, so start at the beginning)
		pz_save_config( ipodRSVP_config );                                 //Save the configuration
		progress = 0;                                                      //Set progress variable to 0 (new book...)
		Update_First_Menu();                                               //Update the menu
	}
	if( key == TTK_BUTTON_ACTION) {
		pz_message ("Selection Confirmed.");                               //Do something if the action button is pressed
	}
	ipodRSVP_saveSettings(this, key, time);                                //While we're at it, save the rest of the settings too
	return ttk_menu_button( this, key, time );
}

static PzWindow *new_displaySpeedWindow()
{ 
	displaySpeedWindow = pz_new_window("Display Speed", PZ_WINDOW_NORMAL);
	pz_add_widget (displaySpeedWindow, draw_displaySpeedWindow, event_displaySpeedWindow);
	return pz_finish_window (displaySpeedWindow);
}

static PzWindow *new_selectBook()
{
	return TTK_MENU_DONOTHING;                                             //We actually don't do anthing here.
	                                                                       //There is a window to hold the list of books, 
	                                                                       //But it is really a setting, not a new window
}
static PzWindow *new_ipodRSVP_window ()
{
  delay = ((int)(1000.0/((float)*value/60)));                              //Calculate the delay in milliseconds based on the value (roughly WPM)
  if(*value == 0){delay = 1000;}
  PzWindow *ret;
  FILE *fp;
  char file[266] = "/mnt/Notes/";                                          //We look for the file in /mnt/Notes
  strcat(file,SelectedBook);                                               //Add filename to the path (i.e. /mnt/Notes/filename.txt)
  fp = fopen (file, "r");                                                  //Open the fully-pathed file for reading
  if (!fp)                                                                 //If file cannot be opened...
    {
      pz_warning ("Could not read from %s: %s.",file,strerror (errno));    //Print warning message telling the user the file wasn't found (this shouldn't happen)
      text = (char *) strdup ("File could not be opened");                 //Set text to an error message
    }
  else                                                                     //If everything worked out...
    {
      long len;
	  char *lf_pos = 0;
      fseek (fp, 0, SEEK_END);                                             //Move to the end of the file
      len = ftell (fp);                                                    //Determine length (in bytes) of file
      if(len){
		  fseek (fp, 0, SEEK_SET);                                         //Move back to the beginning
		  text = malloc (len + 1);                                         //Allocate space for the whole file + 1 (for null terminator)
		  fread (text, 1, len, fp);                                        //Read the entire file into text
		  text[len] = 0;                                                   //Make the last character a 0 to end the string
		  if(progress < 0){progress = progress * -1;}                      //If for some reason progress is negative (this should not happen)
		  lf_pos = strchr (text, '\n');                                    //Locate a new line
		  if(*(lf_pos-1) == 13){unix_or_windows = 1; }                     //If the line feed character is preceded by a CR (ASCII 13), we're dealing with a windows file
		  position = text + progress;                                      //Move the position pointer to the current progress
		  state = 1;                                                       //Begin reading
	  }
	  else{strcpy(currentWord,"Empty File");}
	  
    }

  ret = pz_new_window ("ipodRSVP", PZ_WINDOW_NORMAL);                      //Ready the main window
  text_area = pz_add_widget (ret, draw_ipodRSVP, event_ipodRSVP);          //Set up the widgets
  ttk_create_timer (1500, draw_text);                                      //Initialize the timer (start with a longer delay so the user can get ready)
  return pz_finish_window (ret);                                           //Open the main window
}

static void draw_ipodRSVP (PzWidget * wid, ttk_surface srf)
{
  ttk_font rsvp_Font = ttk_get_font ("BitStreamVeraMono", 16);             //Load the desired font
  
  //Print the current word
  ttk_text (srf, rsvp_Font, wid->x + (wid->w - ttk_text_width (rsvp_Font, currentWord)) / 2,
	    (wid->h/ 2)-ttk_text_height(rsvp_Font),ttk_makecol (BLACK), currentWord);
}



static void draw_displaySpeedWindow (PzWidget *wid, ttk_surface srf)
{
	// Draw the message
	int y = wid->y + 5;
	//Center the text in the window
	ttk_text (srf, ttk_textfont, wid->x + (wid->w - ttk_text_width (ttk_textfont, "Select Display Speed")) / 2, y, ttk_makecol (BLACK), "Select Display Speed");
	y += 3;

	//The slider will be centered and its length is determined by the size of the window
	slide = ttk_new_slider_widget (15, (wid->h/2) - 25, wid->w - 30, 50,1000, value, "");
	ttk_add_widget(displaySpeedWindow,slide);                              //Add slider to the menu
	
	//Add the - and + to either side of the slider
	ttk_text (srf, ttk_menufont, 5,(wid->h/2) - 13, ttk_makecol (BLACK), "-");
	ttk_text (srf, ttk_menufont, wid->w - 10,(wid->h/2) - 13, ttk_makecol (BLACK), "+");
	
	//Center the message at the bottom
	ttk_text (srf, ttk_menufont, wid->x + (wid->w - ttk_text_width (ttk_menufont, "Press Menu to save.")) / 2,
		wid->y + wid->h - ttk_text_height (ttk_menufont) - 5, ttk_makecol (BLACK), "Press Menu to save.");
}

static int event_displaySpeedWindow (PzEvent *ev)
{
   switch (ev->type) {
		case PZ_EVENT_BUTTON_UP:
			pz_close_window (ev->wid->win);
			break;

		case PZ_EVENT_DESTROY:
			// safely release the resources, as they might not have been allocated
			// due to the current configuration (i.e. image may be disabled).
			break;
	}
	return 0;
}


static int event_ipodRSVP (PzEvent * ev)
{
  switch (ev->type)
    {

		case PZ_EVENT_BUTTON_UP:
		  if (ev->arg == PZ_BUTTON_MENU)                                    //If the menu button is pressed...
		  {
			  //Save current progress, and update menus
			  if(state){ttk_destroy_timer(MainTimer);}                     //Stop the timer (if running)
			  state = 0;                                                   //Stop the reading
			  pz_set_int_setting (ipodRSVP_config, BOOKMARK, progress);    //Save progress in the book
			  pz_save_config( ipodRSVP_config );                           //Save configuration
			  Update_First_Menu();                                         //Update the menu
			  free (text);                                                 //Free memory for text
			  pz_close_window (ev->wid->win);                              //Close the window
		  }
		  else if (ev->arg == PZ_BUTTON_PLAY)                              //If the pause/play button is pressed...
		  {
			  if(strcmp(currentWord,"Empty File")==0){break;}              //If it's an empty file for whatever reason, we're done already
			  if (state)                                                   //If current state is playing...
				{
				  state = 0;                                               //Pause playback
				}
			  else                                                         //Otherwise, the current state is paused
				{
				  state = 1;                                               //Resume playback
				  ttk_create_timer (delay, draw_text);                     //Set the timer
				}
				last_direction = 1;                                        //Last direction was forward
		  }
		  else if (ev->arg == PZ_BUTTON_PREVIOUS)                          //If the previous button is pressed...
		  {
			  if(strcmp(currentWord,"Empty File")==0){break;}              //If it's an empty file for whatever reason, we're done already
			  state = 0;                                                   //Stop playback
			  if (last_direction)                                          //If the last direction was forward
				{
				  if(((ttk_get_podversion() <= 20) && long_word == 0)){move_word (0);} //If we're using an older iPod, but this isn't a long word, go back to the previous word (as opposed to the start of the current one)
				  else if((ttk_get_podversion() <= 20) && beginning_of_long_word == long_word){move_word(0);}        //Also, if this IS a long word on an old iPod, but we're at the start of that word, a previous should take us back to the previous word
				  else if(ttk_get_podversion() > 20 ){move_word(0);}       //If we're not using an old iPod, long words don't matter so much
				}
			  move_word (0);                                               //Move backwards one word
			  last_direction = 0;                                          //Set last direction as backwards
			  text_area->dirty = 1;                                        //Redraw the window
		  }
		  else if (ev->arg == PZ_BUTTON_NEXT)                              //If the next button is pressed...
		  {
			  if(strcmp(currentWord,"Empty File")==0){break;}              //If it's an empty file for whatever reason, we're done already
			  if (!last_direction)                                         //If the last direction was backwards
				{
				  move_word (1);                                           //Move forward one word (actually to the current word)
				}
			  move_word (1);                                               //Move forward one word
			  last_direction = 1;                                          //Set last direction as forward
			  text_area->dirty = 1;                                        //Redraw the window
		  }
		  break;
		case PZ_EVENT_DESTROY:                                             //If/when this window is destroyed, free stuff
		  free (text);                                                     //For some reason, I don't think this ever happens
		  free (currentWord);
		  printf("destroy! destroy!!\n");
		break;
    }
  return 0;
}

static void cleanup_ipodRSVP ()
{
	free((char *)SelectedBook);                                            //Free memory for the selected book  
	free (value);                                                          //Free value (everything else has already been freed)
	//printf("cleanup_ipodRSVP\n");
}


static void init_ipodRSVP ()
{
  DIR *notesDIR;                                                           //Hold directory info for text files directory
  struct dirent *currentEntry;                                             //Currently selected file in directory
  char* location;                                                          //Position of found character
  char tempEntry[255] = "/Extras/Utilities/iPod RSVP/Change Book.../";     //Entry for changing the book
  char firstBook[255] = "";                                                //First book in directory (if none selected this will be in the first menu item)
  module = pz_register_module ("ipodrsvp", cleanup_ipodRSVP);              //Register the module with podzilla

  ipodRSVP_config = pz_load_config( pz_module_get_cfgpath(module, "ipodrsvp.conf") );  //Configuration data

  //Set aside space for our variables
  value = malloc(sizeof(int));
  SelectedBook = malloc(300);
  
  //Initialize settings from configuration file
  *value = pz_get_int_setting(ipodRSVP_config, DISPLAY_SPEED);
  SelectedBook = pz_get_string_setting(ipodRSVP_config, CURRENT_FILE);
  progress = pz_get_int_setting(ipodRSVP_config, BOOKMARK);
  
  //Set currentWord to something to begin
  currentWord[0] = 'L'; currentWord[1] = 'o'; currentWord[2] = 'a';
  currentWord[3] = 'd'; currentWord[4] = 'i'; currentWord[5] = 'n';
  currentWord[6] = 'g'; currentWord[7] = '.'; currentWord[8] = '.'; 
  currentWord[9] = '.'; currentWord[10] = 0;
  
  pz_menu_add_action(firstEntry,new_selectBook);                           //Add first menu item (this is temporary)
  
  notesDIR=opendir("/mnt/Notes");                                          //Open the notes directory
  while ((currentEntry=readdir(notesDIR))){                                //Cycle through the files in the directory
	  if(strstr(currentEntry->d_name, ".txt") || strstr(currentEntry->d_name, ".TXT"))  //Only select text files
	  {
		  files_exist = 1;
		  if(strlen(firstBook) == 0){strcpy(firstBook, currentEntry->d_name);}//If there is no entry in first book, make it the first text file 
		  strcat(tempEntry,currentEntry->d_name);                          //Build the current menu item from the text file's name (i.e. /menu/menu/filename.txt)
		  pz_menu_add_action( tempEntry, new_selectBook);                  //Add the current menu item to the menu
		  location = strrchr(tempEntry,'/');                               //Find the last '/' character
		  *(location + 1) = 0;                                             //Truncate the current menu item after the last '/' character
	  }
  }
  closedir(notesDIR);                                                      //Close the directory
  if(strlen(SelectedBook) == 0)                                            //If there is no selected book...
  {
	printf("%s\n",SelectedBook);
    if(files_exist){
		pz_menu_remove (firstEntry);                                       //Remove the top menu item
	    location = strrchr(firstEntry,'/');                                //Find the last '/' character
		*(location + 1) = 0;                                               //End the string after the last '/'
		strcat(firstEntry,"Start ");                                       //Add 'Start' to the beginning of the menu
		pz_set_string_setting (ipodRSVP_config, CURRENT_FILE, firstBook);  //Save the current book's filename
        SelectedBook = pz_get_string_setting(ipodRSVP_config, CURRENT_FILE);//Load it back into SelectedBook
		strcat(firstEntry,SelectedBook);                                   //Add the first book in the list to the menu
		pz_menu_add_top (firstEntry, new_ipodRSVP_window);                 //Add the new top menu item
	}
  }
  else                                                                     //Otherwise...
  {
	if(files_exist){Update_First_Menu();}                                  //Update the menu
  }
  if(files_exist)
  {
	  pz_menu_add_action("/Extras/Utilities/iPod RSVP/Display Speed", new_displaySpeedWindow);	 //Add the item for adjusting the display speed
	  pz_menu_add_option("/Extras/Utilities/iPod RSVP/Pause after.../Commas",0);                 //Add menu items for the various
	  pz_menu_add_option("/Extras/Utilities/iPod RSVP/Pause after.../Misc. punctuation",0);      //situations when pausing might be
	  pz_menu_add_option("/Extras/Utilities/iPod RSVP/Pause after.../End of sentences",0);       //necessary
	  
	  pz_menu_set_option("/Extras/Utilities/iPod RSVP/Pause after.../Commas",pz_get_int_setting(ipodRSVP_config,PAUSE_COMMA));                   //Set the pause options
	  pz_menu_set_option("/Extras/Utilities/iPod RSVP/Pause after.../Misc. punctuation",pz_get_int_setting(ipodRSVP_config,PAUSE_MISC_PUNCT));   //based on the configuration
	  pz_menu_set_option("/Extras/Utilities/iPod RSVP/Pause after.../End of sentences",pz_get_int_setting(ipodRSVP_config,PAUSE_SENTENCE));
	  
	  pz_menu_add_option("/Extras/Utilities/iPod RSVP/Pause Length: ",pause_length);
	  pz_menu_set_option("/Extras/Utilities/iPod RSVP/Pause Length: ",pz_get_int_setting(ipodRSVP_config,PAUSE_LENGTH));
	  
	  
	  // use these callbacks to save our settings
	  ((TWidget *) pz_get_menu_item ("/Extras/Utilities/iPod RSVP")->data)->button = ipodRSVP_saveSettings;
	  ((TWidget *) pz_get_menu_item ("/Extras/Utilities/iPod RSVP/Pause after...")->data)->button = ipodRSVP_saveSettings;
	  ((TWidget *) pz_get_menu_item ("/Extras/Utilities/iPod RSVP/Change Book...")->data)->button = ipodRSVP_saveBook;
  }
}

PZ_MOD_INIT (init_ipodRSVP)
